/*
 * Copyright 2005 - 2009  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * Alfresco Plugin - Alfresco Space Browser JS code
 *
 * Created on 03.11.2008
 * Author: Sebastian Kummer <s.kummer@zarafa.com>
 *
 */

alfrescospacemodule.prototype = new Module;
alfrescospacemodule.prototype.constructor = alfrescospacemodule;
alfrescospacemodule.superclass = Module.prototype;


/**
 * Constructor 
 */ 
function alfrescospacemodule(id, element, title, data) {
    if(arguments.length > 0) {
        this.init(id, element, title, data);
    }
    this.viewFiles = false;    
    this.targetUrl = '';
    this.entryid = "";
    this.parententryid = "";
    this.storeid = "";
}


/**
 * Initialization
 */ 
alfrescospacemodule.prototype.init = function(id, element, title, data) {
    alfrescospacemodule.superclass.init.call(this, id, element, title, data);
}


/**
 * Setter if in filebrowser or spacebroser
 *
 * @param boolean viewFiles
 */ 
alfrescospacemodule.prototype.setFilesView = function(viewFiles) {
    this.viewFiles = viewFiles;
}


/**
 * Setter of targetURL  (used when inserting a Link to an Alfresco File)
 *
 * @param string targetUrl
 */ 
alfrescospacemodule.prototype.setTargetUrl = function(targetUrl) {
    this.targetUrl = targetUrl;
}


/**
 * starts serverside preload of Alfresco nodes
 */
alfrescospacemodule.prototype.preloadAlfrescoData = function(){
        webclient.xmlrequest.addData(this, "preloadData", {});
}


/**
 * Loads the nodes for the browser
 */
alfrescospacemodule.prototype.loadAlfrescoData = function() {
    // reset "founds" counter
    document.getElementById("alfrescoFound").innerHTML = "";
    webclient.xmlrequest.addData(this, "getList", {viewFiles: this.viewFiles});
    webclient.xmlrequest.sendRequest();
}


/**
 * Loads the information about the attachments
 */
alfrescospacemodule.prototype.loadAttachmentsInformation = function(storeid, parententryid, entryid) {
    this.entryid = entryid;
    this.parententryid = parententryid;
    this.storeid = storeid; 
    webclient.xmlrequest.addData(this, "getAttachmentsInformation", {store:storeid, parententryid:parententryid, entryid:entryid});
    webclient.xmlrequest.sendRequest();
}


/**
 * executed when a space or file in the browser was selected
 * calls resultCallBack() function 
 *
 * @param string nodeLink
 * @param string nodeLabel
 */
alfrescospacemodule.prototype.selectAlfrescoNode = function(nodeLink, nodeLabel) {
    if (module.viewFiles == true) {
        var result = {
            nodelink: nodeLink,
            nodelabel: nodeLabel,
            htmllink: null,
            link: null
        }
        if (module.targetUrl) {
            result.htmllink = ' <a href="'+ module.targetUrl + nodeLink +'">'+ nodeLabel +'</a> ';
            result.link = dgettext('plugin_alfresco', 'Link to')+' '+ nodeLabel +'\n'+ module.targetUrl + nodeLink + ' \n';
        }
        window.resultCallBack(result, window.callBackData);
        window.close();
    }
}


/**
 * searches for a string on the Alfresco Server
 */
alfrescospacemodule.prototype.searchAlfresco = function() {
    var searchfor = document.getElementById("searchfor");
    if (searchfor.value.length == 0) return;


    var spaceDiv = document.getElementById("alfrescoSpacesTree");
    if (alTree && spaceDiv) {
        alTree.removeAll();
        alTree.showLoadingNode();
        spaceDiv.innerHTML = alTree;

        // reset "founds" counter
        document.getElementById("alfrescoFound").innerHTML = "";
    }
    webclient.xmlrequest.addData(this, "getList", {searchfor: searchfor.value});
    webclient.xmlrequest.sendRequest();
}


/**
 * empties the search text field
 */
alfrescospacemodule.prototype.clearSearch = function() {
    var searchfor = document.getElementById("searchfor");
    if (searchfor) searchfor.value = "";
    var spaceDiv = document.getElementById("alfrescoSpacesTree");
    if (alTree && spaceDiv) {
        alTree.removeAll();
        alTree.showLoadingNode();
        spaceDiv.innerHTML = alTree;
    }
    this.loadAlfrescoData();
}


/**
 * Function which execute an action. This function is called by the XMLRequest object.
 *
 * @param string type the action type
 * @param object action the action tag 
 */ 
alfrescospacemodule.prototype.execute = function(type, action) {
    switch(type) {
        case "listspaces":
            this.listSpaces(action);
            break;
        case "preLoadFinished":
            this.preLoadFinished(action);
            break;
        case "preLoadError":
            this.preLoadError(action);
            break;
        case "attachments":
            this.listAttachments(action);
            break;
    }
}


/**
 * executed when the server is sending data for the browser
 * existing nodes are deleted (ex. loading node) and received nodes are displayed
 *
 * @param mixed action
 */
alfrescospacemodule.prototype.listSpaces = function(action) {
    var spaceDiv = document.getElementById("alfrescoSpacesTree");

    if (alTree && spaceDiv) {
        // remove the "loading" node
        alTree.hideLoadingNode();

        var imguri = dhtml.getXMLValue(action, "imguri", "none");

        // loop through values from server
        var spaces = action.getElementsByTagName("spaces");
        for(var i=0;i<spaces.length;i++){
            var space = spaces[i];
            var data = {
                space_id: dhtml.getXMLValue(space, "spaceid"),
                parent_id: dhtml.getXMLValue(space, "parentid"),
                display_name: dhtml.getXMLValue(space, "label"),
                title: dhtml.getXMLValue(space, "title"),
                full_path: dhtml.getXMLValue(space, "fullpath"),
                url: null,
                size: this.convertSize(dhtml.getXMLValue(space, "size")),
                isdocument: dhtml.getXMLValue(space, "isdocument"),
                icon: dhtml.getXMLValue(space, "icon")
            }
            // build a clickable Spaces only when not in fileview
            if (data.full_path && (!this.viewFiles || data.isdocument)) 
                data.url = "javascript:alfrescospacemodule.prototype.selectAlfrescoNode('"+data.full_path+"', '"+data.display_name.replace(/'/g, "\\'") +"');";
            if (! data.isdocument) 
                data.size = null;

            if (data.icon) data.icon = imguri+'img/filetypes/' + data.icon +'.gif';
            // add the node to the tree
            alTree.addAlfresco(data);

        }
        spaceDiv.innerHTML = alTree;

        // show the number of documents found
        var found = dhtml.getXMLValue(action, "found", "none");
        if (this.viewFiles)
            found += ' '+ ((found == 1)? dgettext('plugin_alfresco', "file found") : dgettext('plugin_alfresco', "files found") );
        else
            found += ' '+ ((found == 1)? dgettext('plugin_alfresco', "space found") : dgettext('plugin_alfresco', "spaces found") );

        document.getElementById("alfrescoFound").innerHTML = "<nobr>" + found + "</nobr>";
        
        // if performing a search: open the entire tree 
        if (dhtml.getXMLValue(action, "autoexpand", "none") == 'open') 
            alTree.openAll();
    }
}


/**
 * executed when the server is sending data for the browser
 * existing nodes are deleted (ex. loading node) and received nodes are displayed
 *
 * @param mixed action
 */
alfrescospacemodule.prototype.listAttachments = function(action) {

    var attDiv = document.getElementById("attachmentstoarchive");
    
    var attachments = action.getElementsByTagName("attachments");
    
    var value = '<table>';
    for (var i=0; i<attachments.length; i++) {
    	value += '<tr><td style="width: 60px;" valign="top"><input type="checkbox" id="attachment'+i+'" name="attachment" value="attnumber'+ dhtml.getXMLValue(attachments[i], "attachnumber") +'" onclick="filemetainfo('+ dhtml.getXMLValue(attachments[i], 'attachnumber') +',' + attachments.length +', \'cbox\' )" ></td>';
        value += '<td style="width: 100%; padding-top: 5px;" valign="top" ><a href="#" title="' + dgettext('plugin_alfresco', "Enter meta information") + '"  onclick="filemetainfo('+ dhtml.getXMLValue(attachments[i], 'attachnumber') +',' + attachments.length +', \'name\' )">' + dhtml.getXMLValue(attachments[i], "filename") + '</a>';
        value += '<div id="filemetainfo' + dhtml.getXMLValue(attachments[i], "attachnumber") +'" class="filemetainfo"><br>';
        value += '<table class="filemetatable"><tr><td colspan="2"><strong>' + dgettext('plugin_alfresco', "Additional information") + '</strong></td></tr>';
        value += '<tr><td>' + dgettext('plugin_alfresco', "Filename") + ':</td><td style="width: 100%; padding: 0 10px 0 0;"><input id="filemetaname' + dhtml.getXMLValue(attachments[i], 'attachnumber') + '" type="text" value="' + dhtml.getXMLValue( attachments[i], "filename") + '" maxlength="1024" size="35" style="width: 100%;" /></td></tr>';
        value += '<tr><td>' + dgettext('plugin_alfresco', "Title") + ':</td><td style="width: 100%; padding: 0 10px 0 0;"><input id="filemetatitle' + dhtml.getXMLValue(attachments[i], 'attachnumber') + '" type="text" value="' + dhtml.getXMLValue( attachments[i], "filename") + '" maxlength="1024" size="35" style="width: 100%;" /></td></tr>';
        value += '<tr><td valign="middle">' + dgettext('plugin_alfresco', "Description") + ':</td><td style="width: 100%; padding: 0 10px 0 0;"><textarea id="filemetadescr' + dhtml.getXMLValue(attachments[i], 'attachnumber') + '" cols="32" rows="3" style="width: 80%;" ></textarea></td></tr>';
        value += '<tr><td>' + dgettext('plugin_alfresco', "Author") + ':</td><td style="width: 100%; padding: 0 10px 0 0;" ><input id="filemetaauthor' + dhtml.getXMLValue(attachments[i], 'attachnumber') + '" type="text" value="' + dhtml.getXMLValue( attachments[i], "fromname") + '" maxlength="1024" size="35" style="width: 100%;" onclick="clearfield(\'' + dhtml.getXMLValue(attachments[i], 'attachnumber')+'\')" /></td></tr>';
        value += '<tr><td>' + dgettext('plugin_alfresco', "Select author") + ':</td><td style="width: 100%; padding: 0 10px 0 0;" ><input type="radio" id="userauthor' + dhtml.getXMLValue(attachments[i], 'attachnumber') + '" name="userauthor' + dhtml.getXMLValue(attachments[i], 'attachnumber') + '" value="'+escapeHtml(webclient.fullname)+'" onclick="loadauthor(\'userauthor' + dhtml.getXMLValue(attachments[i], 'attachnumber') + '\',\''+ dhtml.getXMLValue(attachments[i], 'attachnumber')+'\');" style="float: left;"  /><span style="float:left; padding: 2px;">'+escapeHtml(webclient.fullname)+'</span> <input type="radio" id="userauthorother' + dhtml.getXMLValue(attachments[i], 'attachnumber') + '" name="userauthor' + dhtml.getXMLValue(attachments[i], 'attachnumber') + '"  value="' + dhtml.getXMLValue( attachments[i], "fromname") + '" onclick="loadauthor(\'userauthorother' + dhtml.getXMLValue(attachments[i], 'attachnumber') + '\',\''+ dhtml.getXMLValue(attachments[i], 'attachnumber')+'\');" checked="checked" style="float: left;" /><span style="float:left; padding: 2px;">' + dhtml.getXMLValue( attachments[i], "fromname") + '</span></td></tr>';
        value += '</table>';
        value += '</div>';
        value += '</td>';
        value += '<td style="width: 60px;  padding-top: 5px;" align="right" valign="top">' + this.convertSize(dhtml.getXMLValue(attachments[i], "attachmentsize")) + '</td>';
        value += '</tr>';
    }
    value += '</table>';
    attDiv.innerHTML = value;
}


/**
* Function to display/hide the table to enter meta information.
*
* @param int attachment - information for which attachment to show
* @param int nrattachments - total number of the attachments of an email
*
* We have to use the quirk with nrattachments because IE doesn't execute
* getElementsByName on div or table tags
* @see http://www.toolazy.me.uk/template.php?content=getelementsbyname.xml
*/
function filemetainfo (attachment, nrattachments, click) {    
    for (var i=0; i<nrattachments; i++) {
        element = document.getElementById("filemetainfo" + i);
        if (element.id == "filemetainfo"+attachment && element.style.display != "block") {
        		if(click == 'cbox'){
        			if(document.getElementById("attachment"+attachment).checked == 1){
        				element.style.display = "block";
        			}
        			else{
        				element.style.display = "none";
        				document.getElementById("attachment"+attachment).checked = 0;
        			}
        		}
        		else if(click == 'name'){
        			element.style.display = "block";
        			document.getElementById("attachment"+attachment).checked = 1;
        		}
        }
        else if (click == 'cbox' && document.getElementById("filemetainfo"+attachment).style.display == "none" && document.getElementById("attachment"+attachment).checked == 0 ){
        	document.getElementById("filemetainfo"+attachment).style.display = "none";
        }
        else {
        	element.style.display = "none";
        }
    }
}

function loadauthor (nameauthor, nfield){
	document.getElementById('filemetaauthor'+nfield).value = document.getElementById(nameauthor).value;
}

function clearfield(field){
	document.getElementById('filemetaauthor'+field).value = "";
	document.getElementById("userauthor"+field).checked = 0;
	document.getElementById("userauthorother"+field).checked = 0;
}

alfrescospacemodule.prototype.archiveFiles = function(alTree) {
    if (!alTree.selectedNode) alert(dgettext('plugin_alfresco', "Select space to archive to."));
    else {
        var selectedNode = alTree.aNodes[alTree.selectedNode];
        var attachments = document.getElementsByName("attachment");
        var attSelected = false;
        var data = new Array;
        for (var i=0; i<attachments.length;i++) {
            if (attachments[i].checked) {
                attSelected = true;
                var fileinfo = {
                    filemetaname: document.getElementById("filemetaname"+i).value,
                    filemetatitle: document.getElementById("filemetatitle"+i).value,
                    filemetadescr: document.getElementById("filemetadescr"+i).value,
                    filemetaauthor: document.getElementById("filemetaauthor"+i).value,
                    filenumber: i,
                }
                
                data[data.length] = fileinfo;
            }
        }
        if(!attSelected) alert (dgettext('plugin_alfresco', "No attachment selected to archive"));
        else {
            data[data.length] = {
                entryid:  this.entryid,
                node: selectedNode.alfrescoID,
                parententryid: this.parententryid,
                storeid: this.storeid,
                
            }
            window.resultCallBack(data, window.callBackData);
            window.close();
        }
    }
}


/**
* Function checks all checkboxes for files to archive
*/
alfrescospacemodule.prototype.selectAllFiles = function() {
    var attachments = document.getElementsByName("attachment");
    for (var i=0; i<attachments.length;i++) {
        attachments[i].checked = true;
    }
}


/**
* Function unchecks all checkboxes for files to archive
*/
alfrescospacemodule.prototype.deselectAllFiles = function() {
    var attachments = document.getElementsByName("attachment");
    for (var i=0; i<attachments.length;i++) {
        attachments[i].checked = false;
    }
}


/**
 * executed when the serverside preload is finished
 *
 * @param mixed action
 */
alfrescospacemodule.prototype.preLoadFinished = function(action) {
    var nodecount = dhtml.getXMLValue(action, "nodecount", "none");
}


/**
 * executed when the serverside preload was not possible
 * popups an error message
 *
 * @param mixed action
 */
alfrescospacemodule.prototype.preLoadError = function(action) {
    var errormsg = dhtml.getXMLValue(action, "errormsg", "none");
    if (errormsg != "none" ) 
        alert(_("Alfresco error")+": "+errormsg);
}


/**
 * converts a size in bytes to a KB/MB/GB/TB string
 *
 * @param integer size in bytes
 * @return string 
 */
alfrescospacemodule.prototype.convertSize = function(size) {
    if (! size) return null;
    
    var nsize = size;
    var type = "B";

    while (nsize > 1023) {
        nsize = nsize / 1024;

        if (type == 'B') { type = 'KB'; }
        else if (type == 'KB') { type = 'MB'; }
        else if (type == 'MB') { type = 'GB'; }
        else if (type == 'GB') { type = 'TB'; break;}
    }
    var str = Math.round(nsize*10)/10+' '+type;
    if (type.length == 1) str += '&nbsp;&nbsp;';
    return str;
}